@@ -6,6 +6,28 @@ namespace xs { namespace unievent { namespace http {
 using namespace panda;
 using xs::protocol::http::strings_to_sv;
 
+struct DelayedResponseGuard {
+    DelayedResponseGuard() noexcept {}
+    DelayedResponseGuard(ServerRequest* request_) noexcept: request{request_}{}
+    DelayedResponseGuard(const DelayedResponseGuard&) = delete;
+    DelayedResponseGuard(DelayedResponseGuard&& other) noexcept { *this = std::move(other); }
+
+    DelayedResponseGuard& operator=(DelayedResponseGuard&& other) noexcept {
+        std::swap(request, other.request);
+        return *this;
+    }
+
+    inline void comply()  noexcept { request = nullptr; }
+
+    ~DelayedResponseGuard() {
+        if (request) {
+            request->drop();
+        }
+    }
+
+    ServerRequest* request = nullptr;
+};
+
 Plack::Plack (bool mp, bool mt) : multiprocess(mp), multithread(mt) {
     psgi_version = Array::create({Simple(1), Simple(1)});
     psgi_errors  = Stash("UniEvent::HTTP::Plack::ErrorHandle").bless(Hash::create());
@@ -27,7 +49,9 @@ void Plack::bind (const ServerSP& server, const Sub& app) {
             respond_now(request, psgi_res);
         }
         else if (psgi_res.is_sub_ref()) {
-            panda::function<Scalar(Array)> fn = [this, request](const Array& psgi_array) -> Scalar {
+            auto guard = DelayedResponseGuard(request);
+            panda::function<Scalar(Array)> fn = [this, request, g = std::move(guard)](const Array& psgi_array) mutable -> Scalar {
+                g.comply();
                 return process_delayed_response(request, psgi_array);
             };
             Sub(psgi_res).call(xs::out(fn));
